package com.dianafisher.chapter1;

import junit.framework.TestCase;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

/**
 * Question 1.6 - Given an image represented by an NxN matrix, where each pixel is 4 bytes,
 *                write a method to rotate the image by 90 degrees.  Write the method in place.
 *
 * Assumptions:
 *      * Each pixel is 4 bytes.
 *      * Clockwise rotation.
 * */

public class RotateImagePixels
{
    public RotateImagePixels()
    {
        try
        {
            loadImage();
        } catch (IOException e)
        {
            e.printStackTrace();
        }
    }

    private void loadImage() throws IOException
    {
        BufferedImage image = ImageIO.read( new File( "ghosts_white.png" ) );
        int height = image.getHeight();
        int width = image.getWidth();
        int[][] pixels = new int[width][height];

        // load rgb data into pixel matrix
        for( int x = 0; x < width; x++ )
            for( int y = 0; y < height; y++ )
                pixels[x][y] = image.getRGB(x,y);

//        // rotate pixels 90 degrees
        rotate( pixels );
//
//        // update the image with the rotated pixel data
        for( int x = 0; x < width; x++ )
            for( int y = 0; y < height; y++ )
                image.setRGB(x, y, pixels[x][y]);


//        int N = 6;
//
//        int[][] test = new int[N][N];
//        int count = 0;
//        for( int i = 0; i < N; i++ ) {
//            for( int j = 0; j < N; j++ ) {
//                test[i][j] = count;
//                count++;
//            }
//        }
//        printMatrix(test);
//        rotate(test);
//        System.out.println("result:");
//        printMatrix(test);

        // write the data back out to a new image file to see the result
        // use png as the image format
        ImageIO.write(image, "png", new File( "ghosts_white_rotated.png"));
    }

    private void printMatrix( int[][] matrix )
    {
        for( int i = 0; i < matrix.length; i++ ){
            System.out.print("[ ");
            for( int j = 0; j < matrix.length; j++ ){
                System.out.print(String.format( "(%d,%d) %2d ", i, j, matrix[i][j] ));
            }
            System.out.println("]");
        }
    }

    public void rotate( int[][] matrix )
    {
        int N = matrix.length;
        System.out.println("N = " + N);
        // Rotate 4 squares at a time, layer-by-layer.
        // There are N/2 layers for a N x N grid.

        // The first is the outer layer, which starts at (0,0) -> (0,1) -> ...-> (0, N-1) horizontally,
        // and (0,0) -> (1,0) -> ... -> (N-1,0) vertically.

        // The next layer starts at (1,1) -> (1, 2) -> ... -> (1, N-2) horizontally,
        // and (1,1) -> (2,1) -> ... -> (N-2, 1) horizontally.

        // Then (2,2) -> (2,3) -> ... -> (2, N-3) and (2,2) -> (3,2) -> ... -> (N-3, 2).
        // etc.

        for( int layer = 0; layer < N/2; layer++ )
        {
            // in each layer, swap 4 cells at a time to perform a 90-degree rotation
            int end = (N-1)- layer;
//            System.out.println(String.format("start = %d, end = %d", start, end));
            // loop from start to end and swap 4 cells at a time
            for( int i = layer; i < end; i++ )
            {
//                System.out.println("i = " + i);
                // hold on to the value of the top cell
                int top = matrix[layer][i];
//                System.out.println(String.format("top = matrix[%d][%d] = %d", layer, i, top));
                // set value of each cell equal to the value from the cell 90-degrees to the left of it
                int left = end - i + layer;
//                System.out.println(String.format("left = matrix[%d][%d] = %d", left, layer, matrix[left][layer]));
//                System.out.println(String.format("bottom = matrix[%d][%d] = %d", end, left, matrix[end][left]));
//                System.out.println(String.format("right = matrix[%d][%d] = %d", i, end, matrix[i][end]));

                matrix[layer][i] = matrix[left][layer];
                matrix[left][layer] = matrix[end][left];
                matrix[end][left] = matrix[i][end];
                matrix[i][end] = top;
            }
//            System.out.println("---------------------");
        }

    }

    public static void main(String[] args)
    {
        new RotateImagePixels();
    }
}
