package com.dianafisher.chapter1;

import junit.framework.TestCase;

import java.util.ArrayList;
import java.util.Arrays;

/**
 * Question 1.3 - given two strings, implement a method to decide if one is a permutation of the other.
 *
 * Assumptions:
 *      * No duplicate elements
 *      * Lower case only
 *      * White space matters - ie. the strings must be of same length to be considered permutations.
 * */

public class StringPermutation extends TestCase
{
    public boolean isPermutation( String str1, String str2 )
    {
        // check that str1 and str2 are the same length
        if( str1.length() != str2.length() ) return false;

        // check if str1 and str2 are the same string
        if( str1.equals( str2 ) ) return true;

        // if not the same string, sort each string in ascending order
        char[] arr1 = str1.toCharArray();
        Arrays.sort(arr1);  // N lgN

        char[] arr2 = str2.toCharArray();
        Arrays.sort(arr2);  // N lgN

        // now that the strings are sorted in ascending order, compare them
        // if the sorted strings are equal, then they are permutations of each other
        String s1 = new String(arr1);
        String s2 = new String(arr2);

        return s1.equals(s2);    // N -> overall (2N log N + N)
    }


    /**
     * optimized version takes 2N
     * */
    public boolean isPermutation_optimized( String str1, String str2 )
    {
        if( str1.length() != str2.length() ) return false;
        if( str1.equals( str2 )) return true;
        else
        {
            // create a boolean array which is the size of the character set (extended ascii assumed here)
            boolean [] arr = new boolean[256];
            // loop through str1 and mark each entry in the array corresponding to each character in the string
            for( int i = 0; i < str1.length(); i++ )
            {
                char ch = str1.charAt(i);
                // set the entry in the boolean array to true
                arr[ch] = true;
            }

            // now loop through the second string and see if it has the same characters
            for( int i = 0; i < str2.length(); i++ )
            {
                char ch = str2.charAt(i);
//                int value = ch - 'a';
                // if this string contains a character which was not already set, bail out and return false since
                // this means we do not have a permutation
                if( !arr[ch] ) return false;
            }
            return true;
        }
    }


    /**
     * returns list of all permutations of a given string
     * */
    public ArrayList<String> getPermutationsOf( String str )
    {
        ArrayList<String> permutationList = new ArrayList<String>();
        permutation( "", str, permutationList );
        return permutationList;
    }

    /**
     * given prefix and string, get permutation
     * */
    private void permutation( String prefix, String str, ArrayList<String> list)
    {
//        System.out.println("prefix = " + prefix + ", str = " + str);
        int length = str.length();
        if( length == 0 ) {
            list.add(prefix);
//            System.out.println(prefix);
        }
        else
        {
            for( int i = 0; i < length; i++ )
            {
                String nextPrefix = prefix + str.charAt(i);
                String nextStr = str.substring( 0, i ) + str.substring( i + 1, length );
//                System.out.println("i = " + i + ", nextPrefix = " + nextPrefix + ", nextStr = " + nextStr );
                 permutation(nextPrefix, nextStr ,list);
            }
        }
    }


    public void test()
    {
        String testString = "snowman";  // 7 characters -> 7! permutations (5040)
        ArrayList<String> list = getPermutationsOf(testString);
//        System.out.println("list.size() = " + list.size());
        for( String s : list )
        {
//            System.out.println(s);
            assertTrue(isPermutation(testString, s));
            assertTrue(isPermutation_optimized(testString, s));
        }

        assertFalse(isPermutation(testString, "def"));
        assertFalse(isPermutation(testString, "adf"));

        // test optimized version
        assertFalse(isPermutation_optimized(testString, "def"));
        assertFalse(isPermutation_optimized(testString, "adf"));
    }
}
